#!/bin/sh

ALLOWRM=1
PATH=$PWD:$PATH
TESTDIR=$PWD

function test_qpidd_start {
  num=1
  LOGDIR="$TESTDIR/log"
  while test -r "$LOGDIR/$num"; do num=$((num+1)); done
  LOGDIR="$LOGDIR/$num"
  export LOGDIR
  mkdir -p $LOGDIR

  export QPID_LOG_ENABLE=debug+
  export QPID_DATA_DIR=$LOGDIR/datadir
  export QPID_AUTH=yes

  echo "Running qpidd broker (see log/$num): "
  qpidd --port 0 >$LOGDIR/qpidd 2>&1 &

  printf "  "

  while
      ! grep "Broker running" $LOGDIR/qpidd 2>/dev/null
  do
      sleep 0.1
  done

  export QPID_PORT=`sed -n '/^[0-9]\+$/p' $LOGDIR/qpidd`
}

function test_qpidd_stop {
  if QPID_PID=`lsof -F p -i :$QPID_PORT`
  then
        QPID_PID=`printf $QPID_PID | cut -b2-`
        kill $QPID_PID && echo ".qpidd killed"
        while ps -p $QPID_PID>/dev/null 2>&1; do
          sleep 0.1
        done
  else
        echo No qpidd found running
        return 1
  fi
}

function test_krb5_kinit {
  printf ".Kinit: "
  if $TESTDIR/krb5/krb5.sh kinit >/dev/null 2>&1; then
    echo "done"
  else
    echo "error"
    mrg_exit "Unable to get Kerberos ticket. Exiting." 1
  fi
}

function testcpp {
  if timeout.sh 2 perftest \
      --port $QPID_PORT \
      -b $FQDN \
      --mechanism GSSAPI \
      --username $USERNAME \
      --tx 1 --count 1 > $LOGDIR/testcpp 2>&1
  then
    return 0
  else
    return 1
  fi
}

function analyzecore {
  for core in core*
  do
    echo "Analyzing file $core..."
    program=`file $core | cut -d\' -f2`
    mv $core $LOGDIR/
    gdb -batch -x gdb.cmd \
      `which $program` \
      $LOGDIR/$core \
      > $LOGDIR/$core.txt \
      2> $LOGDIR/$core.txt-stderr
  done
}

trap "pkill qpidd" 0 1 3
eval `krb5/krb5.sh dumpconfig`
kdestroy
krb5/krb5.sh kinit
ulimit -c unlimited
while true
do
  test_qpidd_start
  testcpp
  test_qpidd_stop
  if
    ls core* 2>/dev/null
  then
    analyzecore
  else
    [ $ALLOWRM -eq 1 ] && rm -rf $LOGDIR
  fi
done
